*-----------------------------------------------------------
* Title      : 2048
* Written by : Blaine Hubbert and Garrett Gautz
* Date       : 5/1/2014
*
* Description: Use WASD to move blocks as far as they will go
*	in that direction.  Combine like blocks to 'add' them.
*	Make a 2048 block to win. To play again, you must restart the simulator.
*
*	Original game made by Gabriele Cirulli
*
*	CAPITAL OPCODED CODE WRITTEN BY BLAINE HUBBERT
*	lowercase opcoded code written by Garrett Gautz
*
*	Random number generator created by Lee Davison:
*		Random by Lee Davison: generates a binary file of 65536 
*		pseudo random bytes which can be analyzed to gauge the 
*		effectiveness of the random number generator.
*
*	Thank you to everyone above (and Charles Kelly) for helping 
*	us accomplish our class project.
*-----------------------------------------------------------
    ORG    $1000
START:                  ; first instruction of program
BEGIN					*infinite loop label
	OPT		MEX
drawnum  MACRO  
	move.l	D0,-(SP)
	move.l	D1,-(SP)
	move.l  D2,-(SP)
	move.l	D3,-(SP)
	move.l  A3,-(SP)
	
    *draws a number on the grid roughly in ARIAL font size 20 pixel by pixel
    *\1 is address of the start of the string containing the numbers needed to draw
    *\2 denotes which place (ones place, tens place, hundreds place, thousands place)
	move.b	#80,D0			set pen color (outline color)
   	move.l	TEXTCOLOR,D1
    trap	#15
    
    move.l	#0,D1       clear D1 and D2 = X,Y for pixels
	move.l	#0,D2
	move	LEFT,D1
    sub		OFFSET,D1   move X,Y to upper-left corner of the box already drawn
    move.b    \2,D0       
    ************* MOVES X COORDINATE OVER TO APPROPRIATE LOCATION **************
    if.l (A5) <LE> #8 then *single digit
	    add		#34,D1		offset from left to middle of square
	    bra     ENDADD\@
	endi
    if.l (A5) <LE> #64 then *double digit
        if.b D0 <EQ> #1 then  *digit one pushed right to its location
        add     #25,D1
        endi
        if.b D0 <EQ> #2 then
	    add     #42,D1    *digit two pushed right to its location
        endi
        bra     ENDADD\@
	endi
    if.l (A5) <LE> #512 then *three digit
        if.b D0 <EQ> #1 then
        add     #19,D1    *digit one pushed right to its location
        endi
        if.b D0 <EQ> #2 then
        add     #35,D1    *digit two pushed right to its location
        endi
        if.b D0 <EQ> #3 then
	    add     #51,D1    *digit three pushed right to its location
        endi
        bra     ENDADD\@
	endi
    if.l (A5) <GE> #1024 then *four digit
        if.b D0 <EQ> #1 then
        add     #7,D1     *digit one pushed right to its location
        endi
        if.b D0 <EQ> #2 then
        add     #25,D1    *digit two pushed right to its location
        endi
        if.b D0 <EQ> #3 then
	    add     #42,D1    *digit three pushed right to its location
        endi
        if.b D0 <EQ> #4 then
        add     #59,D1    *digit four pushed right to its location
        endi
ENDADD\@
	endi
    move	D1,D3       save left edge X coordinate value
	
	add		TOP,D2
	add		#30,D2		offset from top to middle of square
	
    lea		\1,A3
    move.b	#82,D0		draw pixel trap task
    for.b COUNTb = #1 to #20 do
    	while (A3) <NE> #0 do
		add		(A3)+,D1
		trap	#15
		endw    
    move	D3,D1		move X coordinate back to left space
    add		#1,D2		move down a row
    add		#2,A3		increment A3 by 2 bytes = 1 word
    endf  
    move.l  (SP)+,A3
	move.l	(SP)+,D3
	move.l  (SP)+,D2
	move.l	(SP)+,D1
	move.l	(SP)+,D0	  
	ENDM

*********************************************************************************
******************* THIS MOVES NUMBERS IN THE GRID ******************************
*********************************************************************************
	move.b    #16,D0    turn cursor off
	move.b    #0,D1
	trap      #15

    MOVE.B	#8,D0
	TRAP	#15
	MOVE.L	D1,PRNl	;set seed for LAB_RND
	LEA	GRID,A1
	MOVE.B	#16,D7
INIT
	CMP.B	#0,D7
	BEQ	INIT2
	SUB.B	#1,D7
	MOVE.L	#0,(A1)+
	BRA	INIT
INIT2
	JSR	SET_RN
	JSR	SET_RN
CON
	;DISPLAY!
	JSR	DISPLAY
	;DISPLAY!
	MOVE.B	#5,D0
	TRAP	#15
	JSR		CLRSC	clears the screen
	IF.B D1 <EQ> #$57 OR.B D1 <EQ> #$77 THEN
		JSR	MOVEUP
	ELSE
	IF.B D1 <EQ> #$41 OR.B D1 <EQ> #$61 THEN
		JSR	MOVELEFT
	ELSE
	IF.B D1 <EQ> #$53 OR.B D1 <EQ> #$73 THEN
		JSR	MOVEDOWN
	ELSE
	IF.B D1 <EQ> #$44 OR.B D1 <EQ> #$64 THEN
		JSR	MOVERIGHT
	ENDI
	ENDI
	ENDI
	ENDI
	IF.B GAMED <EQ> #1 THEN
		BRA	ENDGAME
	ENDI
	move.l D0,-(SP)	
	move.l D5,-(SP)
	move.l A5,-(SP)
	move.l A6,-(SP)
	LEA	CGRID,A6
	LEA	GRID,A5
	MOVE.B	#16,D5
CLOOP
	CMP.B	#0,D5
	BEQ	ECLOOP
	SUB.B	#1,D5
	MOVE.L	(A5)+,(A6)+
	BRA	CLOOP
ECLOOP	
	JSR	MOVEUP
	CMP.B	#1,D0
	BEQ	CLOOP2
	JSR	MOVELEFT
	CMP.B	#1,D0
	BEQ	CLOOP2
	JSR	MOVERIGHT
	CMP.B	#1,D0
	BEQ	CLOOP2
	JSR	MOVEDOWN
	CMP.B	#1,D0
	BEQ	CLOOP2
	BRA	ENDGAME
CLOOP2
	MOVE.B	#16,D5
	LEA	GRID,A5
	LEA	CGRID,A6
CLOOP3
	CMP.B	#0,D5
	BEQ	ECLOOP2
	SUB.B	#1,D5
	MOVE.L	(A6)+,(A5)+
	BRA	CLOOP3
ECLOOP2
	move.l	(SP)+,A6
	move.l	(SP)+,A5
	move.l	(SP)+,D5
	move.l	(SP)+,D0
	CMP.B	#1,D0
	BNE	CON
	JSR	SET_RN
	BRA	CON 
ENDGAME
	move.b	#80,D0			set pen color (outline color)
   	move.l	TEXTCOLOR,D1
    trap	#15
	move.b	#81,D0			set fill color
   	move.l	TEXTCOLOR,D1
    trap	#15
	MOVE.B	#13,D0
	IF.B GAMED <EQ> #1 THEN
		LEA	WON,A1
	ELSE
		LEA	LOST,A1
	ENDI
	TRAP	#15
    
    SIMHALT             ; halt simulator
LEFT    DC.W    20		draw grid coordinate
TOP     DC.W    20		draw grid coordinate
RIGHT   DC.W    385		draw grid coordinate
BOTTOM  DC.W    25		draw grid coordinate
COUNT   DC.B    1		for-loop iteration counter
COUNTa	DC.B	1		for-loop iteration counter 2
COUNTb	DC.B	1		for-loop iteration counter 3
OFFSET  DC.W    90		draw grid offset
*ENDL    	DC.B	    '   ',0
*NO		DC.B    	'           ',0
BOXCOLOR DC.L	$000000FF	red
GRIDCOLOR DC.L	$00FFFFFF	green
TEXTCOLOR DC.L	$00000000	black
*below is the raw values to draw pixels -- 1,1,1,1,1 draws 5 pixels next to each other 
*                                       -- 2,2,2 draws every other pixel
*                                       -- 1,1,2,1 draws 2 pixels next to each other, skips a pixel, then draws a pixel next to
*                                                  the one previously drawn XX_XX 
*                                       -- zero jumps to the next line
ZERO    DC.W        5,1,1,1,1,0,4,1,1,1,1,1,1,0,3,1,1,4,1,1,0,3,1,6,1,0,2,1,8,1,0,2,1,8,1,0,2,9,1,0,1,1,10,0,1,1,10,0,1,1,10,0,1,1,10,0,1,1,10,0,1,1,10,0,2,9,1,0,2,1,8,1,0,2,1,8,1,0,3,1,6,1,0,3,1,1,4,1,1,0,4,1,1,1,1,1,1,0,5,1,1,1,1,0
ONE     DC.W        8,1,0,8,1,0,7,1,1,0,6,1,1,1,0,4,1,1,1,1,1,0,3,1,1,1,2,1,0,3,1,4,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0,8,1,0
TWO		DC.W	    4,1,1,1,1,0,3,1,1,1,1,1,1,1,0,2,1,6,1,1,0,2,1,7,1,0,1,1,9,1,0,1,1,9,1,1,0,11,1,0,11,1,0,10,1,0,9,1,0,8,1,0,7,1,0,6,1,0,5,1,0,4,1,0,3,1,0,2,1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,0
THREE   DC.W        3,1,1,1,1,1,0,2,1,1,1,1,1,1,1,0,1,1,1,5,1,1,0,1,1,8,1,0,10,1,0,10,1,0,10,1,0,8,1,1,0,5,1,1,1,1,0,5,1,1,1,1,1,0,9,1,1,0,10,1,1,0,11,1,0,11,1,0,11,1,0,10,1,1,0,1,1,8,1,0,1,1,1,6,1,1,0,2,1,1,1,1,1,1,1,1,0,4,1,1,1,1,0
FOUR    DC.W        9,1,0,9,1,0,8,1,1,0,7,1,1,1,0,7,1,1,1,0,6,1,2,1,0,5,1,3,1,0,4,1,4,1,0,4,1,4,1,0,3,1,5,1,0,2,1,6,1,0,2,1,6,1,0,1,1,7,1,0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,0,9,1,0,9,1,0,9,1,0,9,1,0,9,1,0
FIVE    DC.W        2,1,1,1,1,1,1,1,1,1,0,2,1,1,1,1,1,1,1,1,1,0,2,1,0,1,1,0,1,1,0,1,1,0,1,1,2,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,0,1,1,1,6,1,1,0,1,9,1,0,11,1,0,11,1,0,11,1,0,11,1,0,11,1,0,1,10,1,0,1,1,8,1,0,1,1,1,6,1,1,0,2,1,1,1,1,1,1,1,1,0,3,1,1,1,1,1,0
SIX     DC.W        5,1,1,1,1,0,4,1,1,1,1,1,1,0,3,1,6,1,0,3,1,7,1,0,2,1,0,2,1,0,2,0,1,1,4,1,1,1,0,1,1,1,2,1,1,1,1,1,0,1,1,1,1,1,5,1,0,1,1,1,1,7,1,0,1,1,9,1,0,1,1,9,1,0,2,9,1,0,2,1,8,1,0,2,1,8,1,0,3,1,6,1,0,3,1,6,1,0,4,1,1,1,1,1,1,1,0,5,1,1,1,1,0
EIGHT   DC.W        4,1,1,1,1,0,3,1,1,1,1,1,1,1,0,2,1,6,1,0,2,8,0,1,1,8,1,0,1,1,8,1,0,2,8,0,2,1,6,1,0,3,1,1,1,1,1,1,0,3,1,1,1,1,1,1,0,2,1,6,1,0,1,1,8,1,0,1,1,9,1,0,1,10,1,0,1,10,1,0,1,1,9,1,0,1,1,8,1,0,2,1,6,1,0,3,1,1,1,1,1,1,0,4,1,1,1,1,0

GRID	DS.L	16		data storage
CGRID	DS.L	16
WON		DC.B	'You won the game!',0
LOST	DC.B	'You lost the game!',0
GAMED	DC.B	0

		
drawbox	
    move.b	#80,D0			set pen color (outline color)
    move.l	#$00000000,D1	black to show more space between colored box and grid edge
    trap	#15
    move.b	#81,D0			set fill color
    move.l	BOXCOLOR,D1
    trap	#15
	
		move    #87,D0          draw box
        move    LEFT,D1
        move    TOP,D2
        move    RIGHT,D3
        move    BOTTOM,D4
        trap    #15
nobox
        move    OFFSET,D0       move position for next box by #OFFSET pixels
        add     RIGHT,D0        
        move    D0,RIGHT
        move    OFFSET,D0    
        add     LEFT,D0
        move    D0,LEFT
	RTS		

********************************************************************************************

PRNl	DS.L	1

dLOOP
	MOVE.B	#4,D6
DLOOP2
	MOVE.L	(A1),D1
	TRAP	#15
	ADDA	#4,A1
	SUB.B	#1,D6
	CMP.B	#0,D6
	BNE	DLOOP2
	MOVE.B	#6,D0
	MOVE.B	#10,D1
	TRAP	#15
	MOVE.B	#13,D1
	TRAP	#15
	MOVE.B	#3,D0
	SUB.B	#1,D7
	CMP.B	#0,D7
	BNE	dLOOP
	RTS
	
SET_RN
	BSR	LAB_RND
	MOVE.L	PRNl,D7
	ASR.L	#8,D7
	ASR.L	#8,D7
	AND.L	#$0000FFFF,D7
	DIVU	#100,D7
	ASR.L	#8,D7
	ASR.L	#8,D7
	IF.B D7 <LT> #90 THEN
		MOVE.L	#2,D3
	ELSE
		MOVE.L	#4,D3
	ENDI
	BSR	LAB_RND
	MOVE.B	PRNl,D7
	MOVE.B	#0,D6
	MOVE.B	#16,D5
	LEA	GRID,A1
SLOOP
	CMP.B	#0,D5
	BEQ	ENDSLOOP
	SUB.B	#1,D5
	IF.L (A1) <EQ> #0 THEN
		ADD.B	#1,D6
	ENDI
	ADDA	#4,A1
	BRA	SLOOP
ENDSLOOP
	MOVE.B	#16,D5
	LEA	GRID,A1
	IF.B #0 <EQ> D6 THEN
		MOVE.L	#0,D7
	ELSE
		DIVU	D6,D7
		ASR.L	#8,D7
		ASR.L	#8,D7
	ENDI
	ADD.B	#1,D7
	IF.B D6 <EQ> D7 OR.B D6 <LT> D7 THEN
		MOVE.B	D6,D7
	ENDI
	
SLOOP2
	CMP.B	#0,D5
	BEQ	ENDSET
	IF.B #0 <EQ> D7 AND.L (A1) <EQ> #0 THEN
		BRA	ENDSET
	ENDI
	SUB.B	#1,D5
	ADDA	#4,A1
	IF.B #15 <EQ> D5 THEN
		SUBA	#4,A1
	ENDI
	IF.L #0 <EQ> (A1) THEN
		SUB.B	#1,D7
	ENDI
	BRA	SLOOP2
ENDSET	
	MOVE.L	D3,(A1)
	RTS
LAB_RND
	MOVE.L	PRNl(A3),D0		* get current seed longword
	MOVEQ	#$AF-$100,D1		* set EOR value
	MOVEQ	#18,D2			* do this 19 times
Ninc0
	ADD.L	D0,D0			* shift left 1 bit
	BCC.S	Ninc1			* if bit not set skip feedback
	EOR.B	D1,D0			* do Galois LFSR feedback
Ninc1
	DBF	D2,Ninc0		* loop
	MOVE.L	D0,PRNl(A3)		* save back to seed longword
	RTS

MOVEUP
	MOVE.B	#0,D0
	MOVE.B	#4,D7	;counter for col
LOOPU1
	CMP.B	#0,D7
	BEQ	ENDUP
	SUB.B	#1,D7
	MOVE.B	#4,D6	;counter for row
	LEA	GRID,A6
	MOVE.B	#3,D5
	SUB.B	D7,D5
IUL	
	CMP.B	#0,D5
	BEQ	LOOPU2
	SUB.B	#1,D5
	ADDA	#4,A6
	BRA	IUL
LOOPU2
	CMP.B	#0,D6
	BEQ	LOOPU1
	ADDA	#16,A6
	IF.B D6 <EQ> #4 THEN
		SUBA	#16,A6
	ENDI
	SUB.B	#1,D6
	MOVE.L	A6,A5
	MOVE.B	D6,D1
	SUB.B	#1,D1
	MOVE.B	D6,D5	;counter for checker
LOOPU3
	IF.B D5 <EQ> #0 THEN
		MOVE.B	#0,D6
		BRA	LOOPU2
	ENDI
	ADDA	#16,A5
	SUB.B	#1,D5
	CMP.L	#0,(A5)
	BEQ	LOOPU3
	MOVE.L	(A6),D2
	MOVE.L	(A5),D3
	IF.L D2 <EQ> D3 THEN
		ASL.L	#1,D2
		IF.L D2 <EQ> #2048 THEN
			MOVE.B	#1,GAMED
		ENDI
		MOVE.L	D2,(A6)
		MOVE.L	#0,(A5)
		MOVE.B	#1,D0
	ELSE
		IF.L D2 <EQ> #0 THEN
			MOVE.L	(A5),(A6)
			MOVE.L	#0,(A5)
			MOVE.B	#1,D0
			ADD.B	#1,D6
			ADD.B	#1,D5
			IF.B D6 <NE> #4 THEN
				SUBA	#16,A6
			ENDI
		ELSE
			IF.B D5 <EQ> D1 THEN
			ELSE
				MOVE.B	#1,D0
				ADDA	#16,A6
				MOVE.L	(A5),(A6)
				SUBA	#16,A6
				MOVE.L	#0,(A5)
			ENDI	
		ENDI
	ENDI
	CMP.B	#0,D5
	BEQ	LOOPU1
	BRA	LOOPU2
ENDUP
	RTS
MOVEDOWN
	MOVE.B	#0,D0
	MOVE.B	#4,D7	;counter for col
LOOPD1
	CMP.B	#0,D7
	BEQ	ENDDOWN
	SUB.B	#1,D7
	MOVE.B	#4,D6	;counter for row
	LEA	GRID,A6
	ADDA	#48,A6
	MOVE.B	#3,D5
	SUB.B	D7,D5
IDL	
	CMP.B	#0,D5
	BEQ	LOOPD2
	SUB.B	#1,D5
	ADDA	#4,A6
	BRA	IDL
LOOPD2
	CMP.B	#0,D6
	BEQ	LOOPD1
	SUBA	#16,A6
	IF.B D6 <EQ> #4 THEN
		ADDA	#16,A6
	ENDI
	SUB.B	#1,D6
	MOVE.L	A6,A5
	MOVE.B	D6,D1
	SUB.B	#1,D1
	MOVE.B	D6,D5	;counter for checker
LOOPD3
	IF.B D5 <EQ> #0 THEN
		MOVE.B	#0,D6
		BRA	LOOPD2
	ENDI
	SUBA	#16,A5
	SUB.B	#1,D5
	CMP.L	#0,(A5)
	BEQ	LOOPD3
	MOVE.L	(A6),D2
	MOVE.L	(A5),D3
	IF.L D2 <EQ> D3 THEN
		ASL.L	#1,D2
		IF.L D2 <EQ> #2048 THEN
			MOVE.B	#1,GAMED
		ENDI
		MOVE.L	D2,(A6)
		MOVE.L	#0,(A5)
		MOVE.B	#1,D0
	ELSE
		IF.L D2 <EQ> #0 THEN
			MOVE.L	(A5),(A6)
			MOVE.L	#0,(A5)
			MOVE.B	#1,D0
			ADD.B	#1,D6
			ADD.B	#1,D5
			IF.B D6 <NE> #4 THEN
				ADDA	#16,A6
			ENDI
		ELSE
			IF.B D5 <EQ> D1 THEN
			ELSE			
				MOVE.B	#1,D0
				SUBA	#16,A6
				MOVE.L	(A5),(A6)
				ADDA	#16,A6
				MOVE.L	#0,(A5)
			ENDI
		ENDI
	ENDI
	CMP.B	#0,D5
	BEQ	LOOPD1
	BRA	LOOPD2
ENDDOWN
	RTS
MOVERIGHT
	MOVE.B	#0,D0	; bool for not moved
	MOVE.B	#4,D7	;counter for rows
LOOPR1
	CMP.B	#0,D7	;if we are done checking the rows
	BEQ	ENDRIGHT ;end if we are done
	SUB.B	#1,D7	;move counter for next row
	MOVE.B	#4,D6	;counter for cols
	LEA	GRID,A6	;load grid into A6
	ADDA	#12,A6	;move to the end of the row
	MOVE.B	#3,D5	;counter for IRL
	SUB.B	D7,D5	;make the counter the proper number
IRL	
	CMP.B	#0,D5	;if we are done moveing to the proper row
	BEQ	LOOPR2	;leave to check said row
	SUB.B	#1,D5	;move counter down
	ADDA	#16,A6	;move fown row	
	BRA	IRL	;repeat
LOOPR2
	CMP.B	#0,D6	;if we are done checking this row
	BEQ	LOOPR1	;go back to check next row
	SUBA	#4,A6	;move to spot left of me
	IF.B D6 <EQ> #4 THEN
		ADDA	#4,A6	;if on the first spot, stay on first spot
	ENDI
	SUB.B	#1,D6	;move counter
	MOVE.L	A6,A5	;load A6 into A5
	MOVE.B	D6,D1	;put the number of the counter in D1
	SUB.B	#1,D1	;use D1 to check if we are still on the first check spot
	MOVE.B	D6,D5	;counter for checker
LOOPR3
	IF.B D5 <EQ> #0 THEN	;if the checker met the end of the row
		MOVE.B	#0,D6	;we are done with this row
		BRA	LOOPR2	;go back and continue checking
	ENDI
	SUBA	#4,A5	;move address to next in row
	SUB.B	#1,D5	;lower the counter
	CMP.L	#0,(A5)	;if the checker is on a zero
	BEQ	LOOPR3	;go back and check the next
	MOVE.L	(A6),D2	;else load start of check to d2
	MOVE.L	(A5),D3	;and load checker into d3
	IF.L D2 <EQ> D3 THEN	;if checker equal starter
		ASL.L	#1,D2	;double d2
		IF.L D2 <EQ> #2048 THEN
			MOVE.B	#1,GAMED
		ENDI
		MOVE.L	D2,(A6)	;put d2 into start
		MOVE.L	#0,(A5)	;put a 0 into checker
		MOVE.B	#1,D0	;
	ELSE
		IF.L D2 <EQ> #0 THEN
			MOVE.L	(A5),(A6)
			MOVE.L	#0,(A5)
			MOVE.B	#1,D0
			ADD.B	#1,D6
			ADD.B	#1,D5
			IF.B D6 <NE> #4 THEN
				ADDA	#4,A6
			ENDI
		ELSE
			IF.B D5 <EQ> D1 THEN
			ELSE	
				MOVE.B	#1,D0
				SUBA	#4,A6
				MOVE.L	(A5),(A6)
				ADDA	#4,A6
				MOVE.L	#0,(A5)
			ENDI
		ENDI
	ENDI
	CMP.B	#0,D5
	BEQ	LOOPR1
	BRA	LOOPR2
ENDRIGHT
	RTS
MOVELEFT
	MOVE.B	#0,D0
	MOVE.B	#4,D7	;counter for col
LOOPL1
	CMP.B	#0,D7
	BEQ	ENDLEFT
	SUB.B	#1,D7
	MOVE.B	#4,D6	;counter for row
	LEA	GRID,A6
	MOVE.B	#3,D5
	SUB.B	D7,D5
ILL	
	CMP.B	#0,D5
	BEQ	LOOPL2
	SUB.B	#1,D5
	ADDA	#16,A6
	BRA	ILL
LOOPL2
	CMP.B	#0,D6
	BEQ	LOOPL1
	ADDA	#4,A6
	IF.B D6 <EQ> #4 THEN
		SUBA	#4,A6
	ENDI
	SUB.B	#1,D6
	MOVE.L	A6,A5
	MOVE.B	D6,D1
	SUB.B	#1,D1
	MOVE.B	D6,D5	;counter for checker
LOOPL3
	IF.B D5 <EQ> #0 THEN
		MOVE.B	#0,D6
		BRA	LOOPL2
	ENDI
	ADDA	#4,A5
	SUB.B	#1,D5
	CMP.L	#0,(A5)
	BEQ	LOOPL3
	MOVE.L	(A6),D2
	MOVE.L	(A5),D3
	IF.L D2 <EQ> D3 THEN
		ASL.L	#1,D2
		IF.L D2 <EQ> #2048 THEN
			MOVE.B	#1,GAMED
		ENDI
		MOVE.L	D2,(A6)
		MOVE.L	#0,(A5)
		MOVE.B	#1,D0

	ELSE
		IF.L D2 <EQ> #0 THEN
			MOVE.L	(A5),(A6)
			MOVE.L	#0,(A5)
			MOVE.B	#1,D0
			ADD.B	#1,D6
			ADD.B	#1,D5
			IF.B D6 <NE> #4 THEN
				SUBA	#4,A6
			ENDI
		ELSE
			IF.B D5 <EQ> D1 THEN
			ELSE
				MOVE.B	#1,D0
				ADDA	#4,A6
				MOVE.L	(A5),(A6)
				SUBA	#4,A6
				MOVE.L	#0,(A5)
			ENDI
		ENDI
	ENDI
	CMP.B	#0,D5
	BEQ	LOOPL1
	BRA	LOOPL2
ENDLEFT
	RTS
DISPLAY
*********************************************************************************
************ THIS FORMATS AND DISPLAYS THE NUMBERS IN A GRID ********************
*********************************************************************************
	move.l	D0,-(SP)
    move.l	D1,-(SP)
    move.l	D2,-(SP)
    move.l	D3,-(SP)
    move.l	D4,-(SP)
    move.l	D5,-(SP)
    move.l	D6,-(SP)
    move.l	D7,-(SP)
    move.l	A5,-(SP)
    
	lea	GRID,A5
	    
    move    #28,LEFT            re-initialize values
    move    #28,TOP             
    move    #107,RIGHT          to draw red boxes
    move    #107,BOTTOM
  
	for.b COUNT = #1 to #4 do	display columns
		for.b COUNTa = #1 to #4 do	*display rows
		*move.l	(A5),D1         get number of current box
		    *draw number on box
		    *\1 is number drawn
		    *\2 is which number place explained in MACRO further
		    if.l (A5) <EQ> #0 then
		    bsr     nobox
		    endi
			if.l (A5) <EQ> #2 then
			move.l  #$000000FF,BOXCOLOR  red
			bsr		drawbox         draw a box
            drawnum TWO,#1
			endi
			if.l (A5) <EQ> #4 then
			move.l  #$00277FFF,BOXCOLOR  orange
			bsr		drawbox         draw a box
			drawnum	    FOUR,#1
			endi
			if.l (A5) <EQ> #8 then
			move.l  #$0013C9FF,BOXCOLOR  gold
			bsr		drawbox         draw a box
			drawnum	    EIGHT,#1
			endi
			if.l (A5) <EQ> #16 then
			move.l  #$0000FFFF,BOXCOLOR  yellow
			bsr		drawbox         draw a box
			drawnum	   ONE,#1
			drawnum    SIX,#2
			endi
			if.l (A5) <EQ> #32 then
			move.l  #$00008C00,BOXCOLOR  dark green
			bsr		drawbox         draw a box
			drawnum    THREE,#1
            drawnum	   TWO,#2
			endi
			if.l (A5) <EQ> #64 then
			move.l  #$0000FF00,BOXCOLOR  green
			bsr		drawbox         draw a box
			drawnum    SIX,#1
			drawnum    FOUR,#2
			endi
			if.l (A5) <EQ> #128 then
			move.l  #$00FFFF00,BOXCOLOR  cyan
			bsr		drawbox         draw a box
			drawnum    ONE,#1
			drawnum    TWO,#2
			drawnum    EIGHT,#3
			endi
			if.l (A5) <EQ> #256 then
			move.l  #$00B4B400,BOXCOLOR  teal
			bsr		drawbox         draw a box
			drawnum    TWO,#1
			drawnum    FIVE,#2
			drawnum    SIX,#3
			endi
			if.l (A5) <EQ> #512 then
			move.l  #$00FF0000,BOXCOLOR  blue
			bsr		drawbox         draw a box
			drawnum    FIVE,#1
			drawnum    ONE,#2
			drawnum    TWO,#3
			endi
			if.l (A5) <EQ> #1024 then
			move.l  #$00FF00FF,BOXCOLOR  magenta
			bsr		drawbox         draw a box
			drawnum    ONE,#1
			drawnum    ZERO,#2
			drawnum    TWO,#3
			drawnum    FOUR,#4
			endi
			if.l (A5) <EQ> #2048 then
			move.l  #$00FFFFFF,BOXCOLOR  white
			bsr		drawbox         draw a box
			drawnum    TWO,#1
			drawnum    ZERO,#2
			drawnum    FOUR,#3
			drawnum    EIGHT,#4
			endi
			
		add.l	#4,A5       increment A5
		endf	
	move    #28,LEFT            re-initialize values
    move    #107,RIGHT          to draw red boxes on next line at left side
    
	move    OFFSET,D0           drop down to next row by #OFFSET pixels
    add     TOP,D0        
    move    D0,TOP
    move    OFFSET,D0    
    add     BOTTOM,D0
    move    D0,BOTTOM
	endf
	
*********************************************************************************
*********************** THIS DRAWS THE GRID *************************************
*********************************************************************************
    

	move.b	#80,D0			set pen color (outline color)
    move.l	GRIDCOLOR,D1
    trap	#15
    move.b	#81,D0			set fill color
    move.l	GRIDCOLOR,D1
    trap	#15
      
    move    #20,LEFT            re-initialize values
    move    #20,TOP             but swap RIGHT and BOTTOM
    move    #385,RIGHT           to draw horizontal lines
    move    #25,BOTTOM
    
    for.b COUNT = #1 to #5 do   draw horizontal lines
        move    #87,D0          draw lines
        move    LEFT,D1
        move    TOP,D2
        move    RIGHT,D3
        move    BOTTOM,D4
        trap    #15
        
        move    OFFSET,D0       change values to have next
        add     TOP,D0          line move down
        move    D0,TOP
        move    OFFSET,D0    
        add     BOTTOM,D0
        move    D0,BOTTOM
    endf
    
    move    #20,LEFT            re-initialize values
    move    #20,TOP             but swap RIGHT and BOTTOM
    move    #25,RIGHT           to draw vertical lines
    move    #385,BOTTOM
    
    for.b COUNT = #1 to #5 do   draw vertical lines
        move    #87,D0
        move    LEFT,D1
        move    TOP,D2
        move    RIGHT,D3
        move    BOTTOM,D4
        trap    #15
        
        move    OFFSET,D0       change values to have next
        add     LEFT,D0         line move right
        move    D0,LEFT
        move    OFFSET,D0    
        add     RIGHT,D0
        move    D0,RIGHT
    endf
    move.l	(SP)+,A5
    move.l	(SP)+,D7
	move.l	(SP)+,D6
	move.l	(SP)+,D5
	move.l	(SP)+,D4
	move.l	(SP)+,D3
	move.l	(SP)+,D2
	move.l	(SP)+,D1
	move.l	(SP)+,D0
	
    RTS
CLRSC
*********************************************************************************
************************* CLEAR SCREEN ******************************************
*********************************************************************************
	move.l	D0,-(SP)
	move.l	D1,-(SP)
	
	move.b    #11,D0
	move.w    #$FF00,D1
	trap      #15

	move.l	(SP)+,D1
    move.l	(SP)+,D0
	RTS
    END    START        ; last line of source


























*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
